rm(list=ls());
library(parellel)

####################################################################################################
####################################################################################################
#  This function calculates the Trio Transgression Ratio between two parents and one progeny
# P1 is a vector of NA, 0, 1, and 2 of length M
# P2 is a vector of NA, 0, 1, and 2 of length M
# Prog is a vector of NA, 0, 1, and 2 of length M
# It is ESSENTIAL that the order of elements in P1, P2, and Prog are identical.

Trio <- function(P1, P2, Prog) {
  # Binds P1, P2, and Prog into matrix
  # Remove any row with any missing data (NA)
  # return the number of complete rows
  dat = cbind(P1, P2, Prog); dat = dat[complete.cases(dat),]
  n = nrow(dat) 

  # TTR calculates the proportion of impossible genotypes
  TTR = sum(ifelse(dat[,1] == 0 & dat[,2] == 2 & dat[,3] == 0,1,
            ifelse(dat[,1] == 1 & dat[,2] == 2 & dat[,3] == 0,1,
            ifelse(dat[,1] == 2 & dat[,2] == 0 & dat[,3] == 0,1,
            ifelse(dat[,1] == 2 & dat[,2] == 1 & dat[,3] == 0,1,
            ifelse(dat[,1] == 2 & dat[,2] == 2 & dat[,3] == 0,1,
            ifelse(dat[,1] == 0 & dat[,2] == 0 & dat[,3] == 1,1,
            ifelse(dat[,1] == 2 & dat[,2] == 2 & dat[,3] == 1,1,
            ifelse(dat[,1] == 0 & dat[,2] == 0 & dat[,3] == 2,1,
            ifelse(dat[,1] == 0 & dat[,2] == 1 & dat[,3] == 2,1,
            ifelse(dat[,1] == 0 & dat[,2] == 2 & dat[,3] == 2,1,
            ifelse(dat[,1] == 1 & dat[,2] == 0 & dat[,3] == 2,1,
            ifelse(dat[,1] == 2 & dat[,2] == 0 & dat[,3] == 2,1,0))))))))))))) / n
  
  tmp = data.frame(TTR = TTR, ntest = n); return(tmp)
}

####################################################################################################
# This function calculates the Duo Transgression Ratio between one parent and one progeny
# P1 is a vector of NA, 0, 1, and 2 of length M
# Prog is a vector of NA, 0, 1, and 2 of length M
# It is ESSENTIAL that the order of elements in P1 and Prog are identical.

Duo <- function(P1, Prog) {
  # Binds P1 and Prog into matrix
  # Remove any row with any missing data (NA)
  # return the number of complete rows
  dat = cbind(P1, Prog); dat = dat[complete.cases(dat),]  
  n = nrow(dat)

  # DTR calculates the proportion of impossible genotypes
  DTR = sum(ifelse(dat[,1] == 0 & dat[,2] == 2, 1,
            ifelse(dat[,1] == 2 & dat[,2] == 0, 1, 0))) / n
  
  tmp = data.frame(DTR = DTR, ntest = n); return(tmp)
}


####################################################################################################
####################################################################################################
# Generate Fake data for demonstration
# Par = 30,000 x 20
# Prog = 30,000 x 5

Par = matrix(sample(c(0,1,2), 30000*20,replace=T),ncol=20); colnames(Par) = c(1:20)  # m x n_par
Prog = matrix(sample(c(0,1,2), 30000*5,replace=T),ncol=5);colnames(Prog) = c(1:5)    # m x n_prog

####################################################################################################
# This bit is where the magic happens.
# This is a nested set of apply functions, the first one is parallelized.
# Change detectCores()-1 to whatever number of cores you want to use.
# duos is a data.frame() in long format with 4 columns (Par, Prog, DTR, and ntest)

duos <- do.call("rbind",mclapply(1:(ncol(Par)),function(x){ 
    do.call("rbind",lapply(1:ncol(Prog),function(z){
      cbind(Par = colnames(Par)[x], 
            Prog = colnames(Prog)[z], 
            Duo(Par[,x],Prog[,z]))
  }))
},mc.cores = detectCores()-1))

# trios is a data.frame() in long formate with 5 columns (P1, P2, Prog, DTR, and ntest)
# This assumes that individuals can be used as both pollen and egg donor (sire/dame).
# For male and female parents, 'Par' should be replaced with a marker matrix of male  and female parents
# For male and female parents, x and y can be indexed to test either  male or female parents
trios <- do.call("rbind",mclapply(1:ncol(Par),function(x){ 
  do.call("rbind",lapply(x:ncol(Par),function(y){ 
    do.call("rbind",lapply(1:ncol(Prog),function(z){
      cbind(P1 = colnames(Par)[x], 
      P2 =colnames(Par)[y],  
      Prog = colnames(Prog)[z], 
      Trio(Par[,x],Par[,y],Prog[,z]))
    }))
  }))
},mc.cores = detectCores()-1))


####################################################################################################
# This next section is identical to the previous, but this works on Windows.
cl = makeCluster(detectCores()-1)
clusterExport(cl=cl, varlist=c("Par", "Prog","Duo")) # This command makes the function and data available for every core

duos <- do.call("rbind",parLapply(cl,1:(ncol(Par)),function(x){ 
  do.call("rbind",lapply(1:ncol(Prog),function(z){
    cbind(Par = colnames(Par)[x], 
          Prog = colnames(Prog)[z], 
          Duo(Par[,x],Prog[,z]))
  }))
}))
stopCluster(cl)



cl = makeCluster(detectCores()-1)
clusterExport(cl=cl, varlist=c("Par", "Prog","Trio")) # This command makes the function and data available for every core

trios <- do.call("rbind",parLapply(cl,1:ncol(Par),function(x){ 
  do.call("rbind",lapply(x:ncol(Par),function(y){ 
    do.call("rbind",lapply(1:ncol(Prog),function(z){
      cbind(P1   = colnames(Par)[x], 
            P2   = colnames(Par)[y],  
            Prog = colnames(Prog)[z], 
            Trio(Par[,x],Par[,y],Prog[,z]))
    }))
  }))
}))
stopCluster(cl)
